----------------------------------------
-- Extremely Simple Text-mode Dialogs --
----------------------------------------
	 
include graphics.e -- color names
include image.e -- save text image

without warning
    
constant COLORS = 1,  FGND = 1, BKGND = 2,   -- box structure
	 COORDS = 2,  ROW = 1, COL = 2,
	 SIZE   = 3,  HEIGHT = 1, WIDTH = 2, MINH = 4, MINW = 8,    
	 STYLE  = 4,      
	 SHADOW = 5,
	 HINT   = 6,
	 HINTCOLORS = 7,
	 BUFFER = 8
	 
global constant SGL = 1, DBL = 2, DHL = 3, DVL = 4, NONE = 5  -- border styles
constant up = 328, dn = 336, left = 331, right = 333, ENTER = 13, ESC = 27, SP = 32

sequence DefaultBox
	 DefaultBox = {{BRIGHT_WHITE,BLUE},  -- default colors
		       {1,1},                -- coordinates top left corner
		       {5,10},               -- box size
			SGL,                 -- single line border
			1,                   -- light shadow
			"Press any key...",  -- hint text
			{GREEN,BLACK},       -- hint colors
			{}}                  -- buffer for save screen
	    
global procedure printat(sequence coords, sequence text)
 position(coords[1],coords[2])
 puts(1,text)    
end procedure

global procedure printcolor(sequence coords, sequence colors, sequence text)
 text_color(colors[1])
 bk_color(colors[2])
 printat(coords,text)   
end procedure        

global function limit (atom n, sequence limits)
 if n < limits[1] then n = limits[1]
 elsif n > limits[2] then n = limits[2]
 end if
 return n
end function 

global function pad (sequence s, atom width)
while length(s) < width do s = s & ' '
end while
return s
end function

global function delete (sequence s, atom loc, atom len)
return s[1..loc-1] & s[loc+len..length(s)]
end function   

procedure MoveCursor(sequence coords)
position(coords[ROW],coords[COL])    
end procedure                       

function SaveBkgnd(object box)
 cursor(NO_CURSOR)
 box[COORDS] = get_position()
 box[BUFFER] = save_text_image(box[COORDS],box[COORDS]+box[SIZE]+2)
 cursor(UNDERLINE_CURSOR)
 return box
end function                  

global procedure RestoreBkgnd(object box)
  MoveCursor(box[COORDS])
  display_text_image(box[COORDS],box[BUFFER])
end procedure            
-------------------------------< BOX >-----------------------------------
	 
constant Lines = {{'','','','','',''}, -- single line borders
		  {'','','','','',''}, -- double line borders
		  {'','','','','',''}, -- double horizontal
		  {'','','','','',''}, -- double vertical
		  {' ',' ',' ',' ',' ',' '}} -- none
		  
constant Divider = {{'','',''}, -- single style divider line
		    {'','',''},
		    {'','',''}, -- double horizontal
		    {'','',''},
		    {' ',' ',' '}}

constant Shadow = {'','',''}  -- 3 different shadow backgrounds
		  
constant TL = 1, HL = 2, TR = 3, VL = 4, BL = 5, BR = 6

global function DisplayBox(sequence box)
sequence topline, botline, midline, divline, rshad, bshad, char
atom row, col, x, y
 box = SaveBkgnd(box)
 row = box[COORDS][ROW] col = box[COORDS][COL]
 x = box[SIZE][WIDTH] y = box[SIZE][HEIGHT]  
 text_color(box[COLORS][FGND])
 bk_color(box[COLORS][BKGND])
 cursor(NO_CURSOR) 

-- grab the area to be shadowed:  
   rshad = save_text_image({row+1,col+x+2},{row+y+1,col+x+2})
   bshad = save_text_image({row+y+1,col+1},{row+y+1,col+x+1})
     
-- select the border style to use: 
   char = Lines[box[STYLE]]
  
-- build the borders:
   topline = char[TL] & repeat(char[HL],x) & char[TR]
   botline = char[BL] & repeat(char[HL],x) & char[BR]
   midline = char[VL] & repeat(' ',x) & char[VL]
   char = Divider[box[STYLE]]
   divline = char[1] & repeat(char[2],x) & char[3]

-- draw the borders: 
   printat({row,col}, topline)  
   for i = 1 to y do
     printat({row+i,col}, midline)  
   end for          
   printat({row + 2,col}, divline)
   printat({row + y,col}, botline)  

-- draw the shadows:
 if box[SHADOW] > 0 then  
    position(20,1)
    for i = 1 to length(rshad) do
	rshad[i][2] = rshad[i][2] / 16
	rshad[i][1] = Shadow[box[SHADOW]]
    end for  
    for i = 2 to length(bshad[1]) by 2 do
	bshad[1][i] = bshad[1][i] / 16
	bshad[1][i-1] = Shadow[box[SHADOW]]
    end for

    display_text_image({row+1,col+x+2},rshad)
    display_text_image({row+y+1,col+1},bshad)
 end if

 MoveCursor(box[COORDS]) -- restore cursor to start
 cursor(UNDERLINE_CURSOR)  
 return box
end function                                               

------------------------------------------------------------                    
global procedure SetColors(atom fgnd, atom bkgnd)
  DefaultBox[COLORS] = {fgnd,bkgnd}
end procedure                          

global procedure SetSize(atom rows, atom cols)
 DefaultBox[SIZE] = {rows,cols}    
end procedure           

global procedure SetStyle(atom style)
 DefaultBox[STYLE] = style    
end procedure                          

global procedure SetHint(sequence hint)
 DefaultBox[HINT] = hint   
end procedure                          

procedure DisplayHint(sequence hint)
 printcolor({25,2},DefaultBox[HINTCOLORS],repeat(' ',78))
 printcolor({25,3},DefaultBox[HINTCOLORS],hint)    
end procedure                       

procedure DisplayList(sequence coords, sequence colors, sequence list, atom default)
   for i = 1 to length(list) do
     if i = default then
       printcolor(coords+{i,0},{colors[2],colors[1]},list[i])   
     else
       printcolor(coords+{i,0},colors,list[i]) 
     end if  
   end for
end procedure                                                     

function GenericBox (sequence title, sequence msg)
object msgbox
  msgbox = DefaultBox -- copy defaults
  msgbox = DisplayBox(msgbox)                         
  printat(msgbox[COORDS]+{1,1},title)
  DisplayList(msgbox[COORDS]+{2,1},msgbox[COLORS],msg,0)
  return msgbox      
end function

global procedure MessageBox (sequence title, sequence msg)
object box
 box = GenericBox(title,msg)  
end procedure
     
global procedure Notify (sequence title, sequence msg)
object box
  box = GenericBox(title,msg)  
  DisplayHint( "Press any key...")
  while get_key() = -1 do -- wait for keypress 
  end while 
  RestoreBkgnd(box)                                           
end procedure

global function Select (sequence title, atom default, sequence items)
object box, hilite
atom key, selection 
 box = GenericBox(title,items)  
 DisplayHint( "Select with up/dn arrows - press enter - esc to cancel" )
 key = -1 selection = default                                 
 cursor(NO_CURSOR)
 DisplayList(box[COORDS]+{2,1},box[COLORS],items,selection)
 while key != ENTER do 
   key = get_key()        
   if key = up then 
     selection = limit(selection - 1,{1,length(items)})      
   elsif key = dn then 
      selection = limit(selection + 1,{1,length(items)})
  elsif key = left then 
      key = 27
   elsif key = right then 
      key = 13 
   elsif key = ESC then 
      selection = -1 
      exit  
   end if   
   if key != -1 then DisplayList(box[COORDS]+{2,1},box[COLORS],items,selection)
   end if
 end while         
 RestoreBkgnd(box)
 cursor(UNDERLINE_CURSOR)
return selection
end function

global function CheckBox(sequence title, sequence items)
object box, selecteditems   
atom key, selection
 for i = 1 to length(items) do
     items[i] = "[ ] "&items[i]
 end for
 box = GenericBox(title,items)
 DisplayHint("Select with up/dn arrows, toggle with spacebar, enter when finished")    
 key = -1  selection = 1
 MoveCursor(box[COORDS]+{selection+2,2})
 while key != 13 do
       key = get_key()
       if key != -1 then
	  MoveCursor(box[COORDS]+{selection+2,2}) 
       end if
       if key = SP then
	if items[selection][2] = 'X' then
	   items[selection][2] = ' ' 
	else   
	  items[selection][2] = 'X'   
	end if  
	  printcolor(box[COORDS]+{selection+2,1},box[COLORS],items[selection])                               
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = up then 
	  selection = limit(selection - 1,{1,length(items)})      
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = dn then 
	  selection = limit(selection + 1,{1,length(items)})
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = ESC then
	  exit
       end if
 end while                                                                 
 selecteditems = {}
 for i = 1 to length(items) do
     if items[i][2] = 'X' then
	selecteditems = append(selecteditems,items[i][5..length(items[i])]) 
     end if
 end for
 return selecteditems
end function                                                                      

global function RadioBox(sequence title, sequence items)
object box, selecteditems   
atom key, selection
 for i = 1 to length(items) do
     items[i] = "( ) "&items[i]
 end for
 box = GenericBox(title,items)
 DisplayHint("Select with up/dn arrows, toggle with spacebar, enter when finished")    
 key = -1  selection = 1
 MoveCursor(box[COORDS]+{selection+2,2})
 while key != 13 do
       key = get_key()
       if key != -1 then
	  MoveCursor(box[COORDS]+{selection+2,2}) 
       end if
       if key = SP then   
	  if items[selection][2] = 'X' then
	    items[selection][2] = ' '
	  else items[selection][2] = 'X'
	    printcolor(box[COORDS]+{selection+2,1},box[COLORS],items[selection]) 
	  end if 
	  for i = 1 to length(items) do   
	    if i != selection then
	     items[i][2] = ' '-- reset all buttons
	     printcolor(box[COORDS]+{i+2,1},box[COLORS],items[i])
	     end if
	    end for 
	  printat(box[COORDS]+{selection+2,1},items[selection])                               
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = up then 
	  selection = limit(selection - 1,{1,length(items)})      
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = dn then 
	  selection = limit(selection + 1,{1,length(items)})
	  MoveCursor(box[COORDS]+{selection+2,2})
       elsif key = ESC then
	  exit
       end if
 end while                                                                 
 selecteditems = {}
 for i = 1 to length(items) do
     if items[i][2] = 'X' then
	selecteditems = append(selecteditems,items[i][5..length(items[i])]) 
     end if
 end for
 return selecteditems
end function                                                                      

global function ListBox(sequence title, sequence items, atom start)
object box                 
atom key, current, fini, max, itemcount
 box = GenericBox(title,"")    
 key = -1  current = 1 
 itemcount = length(items)         
 max = box[SIZE][1] - 3
 for i = 1 to itemcount do
     items[i] = pad(items[i],box[SIZE][2])
 end for
 fini = itemcount
 fini = limit(fini,{1,itemcount}) 
 fini = limit(fini,{fini,max}) 
 cursor(NO_CURSOR)
 DisplayList(box[COORDS]+{2,1},box[COLORS],items[start..fini],start)
 while key != 27 do
    key = get_key()
    if key  = ESC then exit            
    elsif key = ENTER then
	  exit
    elsif key = up then 
	  current = limit(current - 1,{1,itemcount})
	  if current < start then
	     start = limit(start - 1,{1,itemcount})
	     fini = start + max - 1
	  end if   
	  DisplayList(box[COORDS]+{2,1},box[COLORS],items[start..fini],current-start+1)
	  
    elsif key = dn then         
	  current = limit(current + 1,{1,itemcount})
	  if current > fini then
	  fini = limit(fini + 1,{1,itemcount})
	  start = fini - max + 1 
	  end if    
	  DisplayList(box[COORDS]+{2,1},box[COLORS],items[start..fini],current-start+1)
    end if
 end while        
 cursor(UNDERLINE_CURSOR)
 return items[current]
end function

global function Input (sequence title, sequence prompt, sequence default)
sequence result, pcoords, icoords, box       
atom key,i,imax, imode, max
  box = GenericBox(title,prompt)
  pcoords = get_position()
  icoords = pcoords       
  max = length(default)
  printcolor(icoords,{box[COLORS][2],box[COLORS][1]},default)   
  DisplayHint("Edit this field, press enter when done, or ESC to cancel")
  MoveCursor(icoords)
  result = default
  key = -1 i = 0 imax = length(default) imode = 0
  while key != 13 do
    key = get_key() 
    if key != -1 then
       if key = 13 then exit     
       elsif key = 27 then result = default exit
       elsif key = 8 then -- backspace
	     if i > 0 then            
		result = delete(result,i,1) & ' '
		printat(icoords,result)          
		imax = length(result)
		i = i - 1   
	     end if   
       elsif key = 331 then i = limit(i-1,{0,max}) -- left arrow
       elsif key = 333 then i = limit(i+1,{0,max}) -- right arrow
       elsif key = 327 then i = 0 -- home
       elsif key = 335 then i = max-1 -- end
       elsif key = 338 then     -- INS key
	     imode = not(imode) -- toggle insert mode
	     if imode = 1 then cursor(BLOCK_CURSOR)
	     else cursor(UNDERLINE_CURSOR)
	     end if  
      elsif key = 390 then -- F12 key
	    i = save_screen(0,"DIALOG.BMP")
      elsif key > 31 and key < 129 then
	     
       if imode then
	    i = limit(i+1,{1,max})
	    result[i] = key  
	    else  
	    result = result[1..i] & key & result[i+1..length(result)]
	    result = result[1..max]
	    i = limit(i+1,{1,max})
	    end if
     end if    
       printcolor(icoords,{box[COLORS][2],box[COLORS][1]},result)
       position(icoords[1],icoords[2]+i)
   end if
  end while 
  RestoreBkgnd(box)
return result
end function

global procedure Progress(sequence title, sequence msg, atom percent)
object box, pbar
 box = GenericBox(title,{msg})
 pbar = repeat('',box[SIZE][COL] - 4)
 MoveCursor(box[COORDS]+{5,2})
 puts(1,pbar)                       
 pbar = repeat('',length(pbar) * percent * .01)
 MoveCursor(box[COORDS]+{5,2})
 puts(1,pbar)
end procedure

global procedure Delay(atom sec)
object start, fini
 start = time()    
 fini = start + sec
 while time() < fini do
 end while
end procedure

-- In Euphoria 2.1 this file (unless modified) is FREE - 0 statements.
with 360556122 -- delete this statement if it causes an error

